"""Information about Ubuntu support."""

import csv
import http.client
from urllib import request

SUPPORT_INFO_URL = (
    "https://git.launchpad.net/ubuntu/+source/distro-info-data/plain/ubuntu.csv"
)


def get_distro_info() -> dict[str, dict[str, str]]:
    response: http.client.HTTPResponse = request.urlopen(SUPPORT_INFO_URL)
    if response.status != 200:
        raise ConnectionError(response.status)
    reader = csv.DictReader(response.read().decode().splitlines())
    series = {}
    for row in reader:
        version = row["version"].removesuffix(" LTS")
        series[version] = {
            "distribution": "ubuntu",
            "version": version,
            "begin_support": row["release"],
            "end_support": row["eol-server"],
            "begin_dev": row["created"],
            "end_extended_support": row["eol-esm"],
        }
    return series



import datetime
import pytest
from distro_support._distro import SupportRange
from distro_support.errors import NoDevelopmentInfoError, NoESMInfoError


BASIC_DISTRO = SupportRange(
    distribution="basic",
    version="1",
    begin_support=datetime.date(2000, 1, 1),
    end_support=datetime.date(2000, 12, 31),
)
FULL_DISTRO = SupportRange(
    distribution="full",
    version="2",
    begin_support=datetime.date(2000, 1, 1),
    end_support=datetime.date(2000, 12, 31),
    begin_dev=datetime.date(1999, 1, 1),
    end_extended_support=datetime.date(2001, 12, 31),
)
SUPPORT_DATES = [
    (
        datetime.date(1958, 7, 2),
        False,
        False,
        False,
    ),
    (datetime.date(1999, 12, 31), False, False, True),
    (
        datetime.date(2000, 1, 1),
        True,
        False,
        False,
    ),
    (
        datetime.date(2000, 2, 21),
        True,
        False,
        False,
    ),
    (datetime.date(2000, 12, 31), True, False, False),
    (datetime.date(2001, 1, 1), False, True, False),
    (datetime.date(2038, 1, 19), False, False, False),
]


@pytest.mark.parametrize(("date", "supported", "esm", "in_dev"), SUPPORT_DATES)
def test_is_supported_on(date, supported, esm, in_dev):
    assert BASIC_DISTRO.is_supported_on(date) == supported
    if not supported:
        assert not BASIC_DISTRO.is_supported_on(date)

    assert FULL_DISTRO.is_supported_on(date) == supported
    assert FULL_DISTRO.is_supported_on(date, include_esm=True) == supported or esm


@pytest.mark.parametrize(("date", "supported", "esm", "in_dev"), SUPPORT_DATES)
def test_is_esm_on(date, supported, esm, in_dev):
    with pytest.raises(NoESMInfoError):
        BASIC_DISTRO.is_esm_on(date)
    assert FULL_DISTRO.is_esm_on(date) == esm


@pytest.mark.parametrize(("date", "supported", "esm", "in_dev"), SUPPORT_DATES)
def test_is_dev_on(date, supported, esm, in_dev):
    with pytest.raises(NoDevelopmentInfoError):
        BASIC_DISTRO.is_in_development_on(date)
    assert FULL_DISTRO.is_in_development_on(date) == in_dev



import importlib
import json
import pathlib
from distro_support._distro import SupportRange
from distro_support.errors import UnknownDistributionError, UnknownVersionError


def get_support_range(
    distribution: str, version: str, *, get_online: bool = False
) -> SupportRange:
    distro_file = pathlib.Path(__file__).with_name(f"{distribution}.json")
    if not distro_file.exists():
        raise UnknownDistributionError(distribution)

    distro_data = json.loads(distro_file.read_text())
    if version in distro_data:
        return SupportRange(**distro_data[version])
    if not get_online:
        raise UnknownVersionError(distribution, version)

    distro_mod = importlib.import_module(f"{__package__}.{distribution}")

    distro_data = distro_mod.get_distro_info()
    if version in distro_data:
        return SupportRange(**distro_data[version])
    raise UnknownVersionError(distribution, version)



"""Update distribution support data."""

import json
import pathlib

from distro_support import ubuntu


def update_ubuntu():
    ubuntu_data = pathlib.Path(ubuntu.__file__).with_suffix(".json")
    ubuntu_data.write_text(json.dumps(ubuntu.get_distro_info(), indent="  "))


if __name__ == "__main__":
    update_ubuntu()



import dataclasses
import datetime
from typing import Self

from distro_support.errors import NoDevelopmentInfoError, NoESMInfoError


@dataclasses.dataclass(kw_only=True, slots=True)
class SupportRange:
    """A date range representing the support of a Linux distribution."""

    distribution: str
    version: str
    begin_support: datetime.date
    end_support: datetime.date
    begin_dev: datetime.date | None = None
    end_extended_support: datetime.date | None = None

    def is_supported_on(
        self, date: datetime.date, *, include_esm: bool = False
    ) -> bool:
        """Determine whether this item is supported on the given date.

        :param date: The date on which to check support.
        :param include_esm: Whether to include ESM support (default: False)
        :returns: Whether the product is supported on this date.
        """
        if self.begin_support <= date <= self.end_support:
            return True
        if not include_esm or self.end_extended_support is None:
            return False
        if self.begin_support <= date <= self.end_extended_support:
            return True
        return False

    def is_in_development_on(self, date: datetime.date) -> bool:
        """Determine whether this item is still in development on the given date."""
        if self.begin_dev is None:
            raise NoDevelopmentInfoError(self)
        return self.begin_dev <= date < self.begin_support

    def is_esm_on(self, date: datetime.date) -> bool:
        if self.end_extended_support is None:
            raise NoESMInfoError(self)
        return self.end_support < date <= self.end_extended_support

    @classmethod
    def from_json(cls, data: dict[str, str]) -> Self:
        return cls(
            distribution=data["distribution"],
            version=data["version"],
            begin_support=datetime.date.fromisoformat(data["begin_support"]),
            end_support=datetime.date.fromisoformat(data["end_support"]),
            begin_dev=None
            if "begin_dev" not in data
            else datetime.date.fromisoformat(data["begin_dev"]),
            end_extended_support=None
            if "end_extended_support" not in data
            else datetime.date.fromisoformat(data["begin_dev"]),
        )



from __future__ import annotations
import typing

if typing.TYPE_CHECKING:
    from distro_support._distro import SupportRange


class DistroSupportError(Exception):
    """Base error for the distro-support package."""


class NoDevelopmentInfoError(DistroSupportError):
    """User made an action that required development information we don't have."""

    def __init__(self, distro: SupportRange) -> None:
        super().__init__(
            f"No development information for {distro.distribution} {distro.version}"
        )
        self.distro = distro


class NoESMInfoError(DistroSupportError):
    """User made an action that required development information we don't have."""

    def __init__(self, distro: SupportRange) -> None:
        super().__init__(
            f"No extended support information for {distro.distribution} {distro.version}"
        )
        self.distro = distro


class UnknownDistributionError(DistroSupportError):
    def __init__(self, distro: str) -> None:
        super().__init__(f"Unknown distribution '{distro}'")


class UnknownVersionError(DistroSupportError):
    def __init__(self, distro: str, series: str) -> None:
        super().__init__(f"Unknown version {distro} {series}")
